import React, { useState, useEffect } from 'react';
import { getHandleValue } from '../utils/utils.js';
import { usePicker } from '../context.js';
const handleStyle = (isSelected) => {
    return {
        boxShadow: isSelected ? '0px 0px 5px 1px rgba(86, 140, 245,.95)' : '',
        border: isSelected ? '2px solid white' : '2px solid rgba(255,255,255,.75)',
    };
};
export const Handle = ({ left, i, setDragging, setInFocus, }) => {
    const { setSelectedColor, selectedColor, squareSize } = usePicker();
    const isSelected = selectedColor === i;
    const leftMultiplyer = (squareSize - 18) / 100;
    const handleDown = (e) => {
        e.stopPropagation();
        setSelectedColor(i);
        setDragging(true);
    };
    const handleFocus = () => {
        setInFocus('gpoint');
        setSelectedColor(i);
    };
    const handleBlur = () => {
        setInFocus(null);
    };
    return (React.createElement("div", { tabIndex: 0, onBlur: handleBlur, onFocus: handleFocus, id: `gradient-handle-${i}`, onMouseDown: (e) => handleDown(e), style: { left: (left || 0) * leftMultiplyer }, className: "rbgcp-gradient-handle-wrap" },
        React.createElement("div", { style: handleStyle(isSelected), className: "rbgcp-gradient-handle" }, isSelected && (React.createElement("div", { style: {
                width: 5,
                height: 5,
                borderRadius: '50%',
                background: 'white',
            } })))));
};
const GradientBar = () => {
    const { currentColor, addPoint, colors, value, handleGradient, squareSize, deletePoint, isGradient, selectedColor, inFocus, setInFocus, } = usePicker();
    const [dragging, setDragging] = useState(false);
    function force90degLinear(color) {
        return color.replace(/(radial|linear)-gradient\([^,]+,/, 'linear-gradient(90deg,');
    }
    useEffect(() => {
        const selectedEl = window?.document?.getElementById(`gradient-handle-${selectedColor}`);
        if (selectedEl) {
            selectedEl.focus();
        }
    }, [selectedColor]);
    const stopDragging = () => {
        setDragging(false);
    };
    const handleDown = (e) => {
        if (!dragging) {
            addPoint(e);
            setDragging(true);
        }
    };
    const handleMove = (e) => {
        if (dragging) {
            handleGradient(currentColor, getHandleValue(e));
        }
    };
    const handleKeyboard = (e) => {
        if (isGradient) {
            if (e.keyCode === 8) {
                if (inFocus === 'gpoint') {
                    deletePoint();
                }
            }
        }
    };
    const handleUp = () => {
        stopDragging();
    };
    useEffect(() => {
        window.addEventListener('mouseup', handleUp);
        window?.addEventListener('keydown', handleKeyboard);
        return () => {
            window.removeEventListener('mouseup', handleUp);
            window?.removeEventListener('keydown', handleKeyboard);
        };
    });
    return (React.createElement("div", { style: {
            width: '100%',
            marginTop: 17,
            marginBottom: 4,
            position: 'relative',
        }, id: "gradient-bar" },
        React.createElement("div", { style: {
                width: squareSize,
                height: 14,
                backgroundImage: force90degLinear(value),
                borderRadius: 10,
            }, onMouseDown: (e) => handleDown(e), onMouseMove: (e) => handleMove(e) }),
        colors?.map((c, i) => (React.createElement(Handle, { i: i, left: c.left, key: `${i}-${c}`, setInFocus: setInFocus, setDragging: setDragging })))));
};
export default GradientBar;
